package api;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import controlador.GeoConexion;
import modelo.PuntoGeografico;






/**
 * Servlet implementation class GeoApi
 */
@WebServlet({ "/GeoApi", "/punto/*", "/guardar", "/buscar_puntos" })
public class GeoApi extends HttpServlet {
	private static final long serialVersionUID = 1L;
	//El valor del api key
	private String apiKey = "pqntslc";
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public GeoApi() {
        super();
        
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		//Para la validacion del api key
		if(!request.getHeader("apiKey").equals(this.apiKey))
			response.sendError(401, "El api key es incorrecto");
		
		String url = request.getRequestURI();

		String[] partes = url.split("/");

		//Si entra por la URL /buscar_puntos
		if(partes[2].equals("buscar_puntos")){
			double latitud = Double.parseDouble(request.getParameter("lat"));
			double longitud = Double.parseDouble(request.getParameter("lon"));

			GeoConexion geo = new GeoConexion();
			
			PuntoGeografico[] puntos= geo.buscarPuntosCercanos(latitud, longitud);
			
			if(puntos!=null) {
				
				PrintWriter respuesta = response.getWriter();	
				
				/**
				 * [{"latitud":12,"longitud":12,"nombre":"rami","distancia":151},{"latitud":12,"longitud":12,"nombre":"rami","distancia":151}]
				 */
				
				//el String jsonReturn contendrá el json que se va a retornar.
				//este se va armando con los datos del array que se obtuvo de
				//GeoConexion
				String jsonReturn = "";
				jsonReturn += "[";	
				
				for(int i=0; i<puntos.length; i++) {
					jsonReturn += "{";
					jsonReturn += "\"latitud\": " + puntos[i].getLatitud() + ",";
					jsonReturn += "\"longitud\":" + puntos[i].getLongitud() + ",";
					jsonReturn += "\"nombre\":" +"\""+ puntos[i].getNombre() +"\""+ ",";
					jsonReturn += " \"distancia\":" + puntos[i].getDistancia();
					jsonReturn += "}";					
					
					
					if(i<puntos.length-1) {
						jsonReturn += ",";
					}					
				}
				
				jsonReturn += "]";
				System.out.println(jsonReturn);		
				
				response.setContentType("application/json");
				
				respuesta.print(jsonReturn);				
			}
		}
		
		
		
		//Si entra por la URL /punto/*
		else if(partes[2].equals("punto")){
			//Obtiene los parámetros de más en el url, antecedidos
			//por una /			
			//Se extrae la id, que se encuentra luego de la /
			int id =Integer.parseInt(request.getPathInfo().split("/")[1]) ;
			
			GeoConexion geo = new GeoConexion();
			
			PuntoGeografico punto = geo.buscarPunto(id);

			/*
			 * En caso de que no se recupere ningun valor de la base de datos
			 * el nombre será nulo, entonces se envia un 404
			if(punto.getNombre()==null){
				response.sendError(404,"No se recuperó nada kp");
			}
			*/

			
			String jsonReturn = "";
			
			jsonReturn += "{";
			jsonReturn += "\"latitud\": " + punto.getLatitud() + ",";
			jsonReturn += "\"longitud\":" + punto.getLongitud() + ",";
			jsonReturn += "\"nombre\":" +"\""+ punto.getNombre() +"\"";
			jsonReturn += "}";
			
			System.out.println(jsonReturn);
			
			response.setContentType("application/json");
			
			PrintWriter respuesta = response.getWriter();
			
			respuesta.print(jsonReturn);
		}
		else {
			response.getWriter().write("usted ha entrado en un lugar sin datos jeje");
		}
		
		
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//Para la validacion del api key
		if(!request.getHeader("apiKey").equals(this.apiKey))
			response.sendError(401, "El api key es incorrecto");
		
		// TODO Auto-generated method stub
		response.setContentType("application/json");
		
		PrintWriter resp = response.getWriter();
		
		//Extrae los datos que vienen en el request y los guarda en variables
		double latitud = Double.parseDouble(request.getParameter("lat"));
		double longitud = Double.parseDouble(request.getParameter("lon"));
		String nombre = request.getParameter("nombre");
		
		//Se instancia una objeto GeoConexion, que es la clase que posee comunicación con
		//la base de datos
		GeoConexion gc = new GeoConexion();
		
		//Se agrega el punto, lo que retorna su id de la base de datos;
		int identificadorPunto = gc.agregarPunto(latitud, longitud, nombre);
		
		String jsonReturn = "{\"identificador\":"+ identificadorPunto +"}";
		
		/**
		 * {"latitud":1234,"longitud":1234,"nombre":"roshka"}
		 */		
		System.out.println(jsonReturn);
		resp.write(jsonReturn);
		
		System.out.println(identificadorPunto);
		System.out.println(request.getParameter("lat"));
		System.out.println(request.getParameter("lon"));
		System.out.println(request.getParameter("nombre"));
	}

	/**
	 * @see HttpServlet#doDelete(HttpServletRequest, HttpServletResponse)
	 */
	protected void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//Para la validacion del api key
		if(!request.getHeader("apiKey").equals(this.apiKey))
			response.sendError(401, "El api key es incorrecto");
				
		System.out.println("Ejecutando el delete");
		
		//Extrae los parámetros del URL. ya que el url que llama a este servlet es
		//  /punto/* entonces los parámetros entran por /* que es un wildcard
		String pathInfo = request.getPathInfo();
		
		//Con este se quitan las barras y se escoge la segunda posición (1)
		//del array, porque en ese lugar queda, en este caso, la información
		//que nos interesa, o sea el id
		//Se parese a integer porque inicialmente está como String
		int id = Integer.parseInt(pathInfo.split("/")[1]);
		
		
		//Se setea la salida para que sea interpretada como JSON
		response.setContentType("application/json");
		
		//Se instancia un objeto de GeoConexion, la clase que contiene
		//todos los métodos que se comunica con la base de datos
		GeoConexion geo = new GeoConexion();		
		
		//el método geo.borrarPunto(id) retorna un booleano
		String jsonReturn = "{\"isDelete\":" + geo.borrarPunto(id)+"}";		
		
		PrintWriter respuesta = response.getWriter();
		respuesta.print(jsonReturn);
		
		
	}

}
